@@ -697,7 +697,6 @@ hal_property_modified (LibHalContext *ctx __attribute__((__unused__)),
 	}	
 }
 
-
 /** Invoked when a device in the GDL emits a condition that cannot be
  *  expressed in a property (like when the processor is overheating)
  *
@@ -737,7 +736,6 @@ hal_device_condition (LibHalContext *ctx __attribute__((__unused__)),
 	}
 }
 
-
 /** Invoked by libhal for integration with our mainloop. 
  *
  *  @param  ctx                 LibHal context
@@ -750,6 +748,47 @@ hal_mainloop_integration (LibHalContext *ctx __attribute__((__unused__)),
 	dbus_connection_setup_with_g_main (dbus_connection, NULL);
 }
 
+/** Internal HAL initialization function
+ *
+ * @functions			The LibHalFunctions to register as callbacks.
+ * @return			The LibHalContext of the HAL connection or
+ *				NULL on error.
+ */
+static LibHalContext *
+gvm_do_hal_init (LibHalFunctions *functions)
+{
+	LibHalContext *ctx;
+	char **devices;
+	int nr;
+
+	ctx = hal_initialize (functions, FALSE);
+	if (!ctx) {
+		warn ("failed to initialize HAL!\n");
+		return NULL;
+	}
+
+	if (hal_device_property_watch_all (ctx)) {
+		warn ("failed to watch all HAL properties!\n");
+		hal_shutdown (ctx);
+		return NULL;
+	}
+
+	/*
+	 * Do something to ping the HAL daemon - the above functions will
+	 * succeed even if hald is not running, so long as DBUS is.  But we
+	 * want to exit silently if hald is not running, to behave on
+	 * pre-2.6 systems.
+	 */
+	devices = hal_get_all_devices (ctx, &nr);
+	if (!devices) {
+		warn ("seems that HAL is not running\n");
+		hal_shutdown (ctx);
+		return NULL;
+	}
+	hal_free_string_array (devices);
+
+	return ctx;
+}
 
 int
 main (int argc, char *argv[])
@@ -782,16 +821,9 @@ main (int argc, char *argv[])
 	gtk_signal_connect (GTK_OBJECT (client), "die",
 			    GTK_SIGNAL_FUNC (gtk_main_quit), NULL);
 
-	hal_ctx = hal_initialize (&hal_functions, FALSE);
-	if (!hal_ctx) {
-		warn ("failed to initialize HAL!\n");
+	hal_ctx = gvm_do_hal_init (&hal_functions);
+	if (!hal_ctx)
 		return 1;
-	}
-
-	if (hal_device_property_watch_all (hal_ctx)) {
-		warn ("failed to watch all HAL properties!\n");
-		return 1;
-	}
 
 	gvm_init_config ();
 
